/*=============================================================================
	Copyright (c) 1999-2004 pmG Worldwide, LLC.

	File:	messiah_access.h
	Author:	Lyle Milton
	Date:	01/01/03

		last modified: 04/26/04 - LM
			- added weight tool
			- added world object


	------------------------------------------------------------
	REQUESTING ACCESS
	------------------------------------------------------------

	The access levels specify the stages during the operation
	of the program.  The entry points specify the actual entry
	points for each access level.

	For example, if you want your effect or shader module to be
	called during the module initilization stage, you'd have to
	first request that access when you register your module.

	EX1: Requesting Access

		_pluginEntry
		{
			-
			-
			-
			fxModuleAccess(ACCESS_SYSTEM, S_MODULE_INIT, 0);
			fxModuleRegister(FX_MODTYPE_SHADER, "TestShaderModule", &TestShaderAccess, 0);
			-
			-
			-


	In this way you could request access for different levels
	when registering your module.

	EX2: Requesting Multip Access for Multiple Modules

			-
			-
			-
			fxModuleAccess(ACCESS_OBJECT, O_CREATE|O_DESTROY|O_LOAD|O_SAVE, 0);
			fxModuleRegister(FX_MODTYPE_SHADER, "MyShaderModule", &ShaderAccess, 0);
			-
			-
			-
			fxModuleAccess(ACCESS_SYSTEM, S_MODULE_INIT, 0);
			fxModuleAccess(ACCESS_OBJECT, O_CREATE|O_DESTROY|O_LOAD|O_SAVE, 0);
			fxModuleAccess(ACCESS_PROCESS, P_POST_POINT_DISPLACE, 0);
			fxModuleRegister(FX_MODTYPE_EFFECT, "MyEffectModule", &EffectAccess, 0);
	


	Once you've requested access, you can respond to the given
	messages, at the given access levels.

	EX3: Responding to access messages

		FX_ACCESSFUNC(EffectAccess)
		{
			switch(level)
			{
				case ACCESS_OBJECT:
					switch(entry)
					{
						case O_SAVE:
						-
						-
						-
						case O_CREATE:
							-
							-
							-
				case ACCESS_SYSTEM:
					switch(entry)
					{
						case S_MODULE_INIT:
							-
							-
							-

	------------------------------------------------------------
	IMPORTANT NOTE:  If you set cases for particular access in 
	your Access callback, and you never receive messages for that
	acces, this may mean that level of access is not applicable to
	your specific module type.

	For example, if you set your effect module to receive WT_*
	access messages, you'll never get called because that particular
	level of access doesn't apply to effects.

	However, if you're sure that your module should be receiving
	a particular level of access, and your module isn't being called
	*ALWAYS CHECK YOUR REQUESTED ACCESS FIRST*.  You will never
	receive a message that you hadn't requested in your fxModuleAccess
	call(s) in your _pluginEntry.



	------------------------------------------------------------
	GETTING/SETTING ACCESS DATA
	------------------------------------------------------------
	Some messages may also send data via the FX_AccessInfo *ai
	param of your Access func.  You must use the fxArg* funcs to
	get/set this data.  Not all messages have corresponding data
	so check the relevant messages/files for additional information.


=============================================================================*/


#ifndef _MESSIAH_ACCESS
#define _MESSIAH_ACCESS


//Argument Indentifier,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
typedef struct FX_Arg_St FX_Arg;

//Argument Format,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
typedef struct FX_Format_St FX_Format;

//Access info,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
typedef struct FX_AccessInfo_St FX_AccessInfo;


#define FX_ACCESSFUNC(f)			FXintf f(FX_AccessInfo *ai, FXentity ID, FXint level, FXint64 entry)
#define FX_ENTRYFUNC(f)				FXintf f(FX_AccessInfo *ai, FXentity ID, FXint64 entry)



//Entry Values-----------------------------------------------------------------
#define BREAK_ENTRY					1
#define REPLACE_ENTRY				1
#define CONTINUE_ENTRY				0
#define OK_ENTRY					0

#define	ENTRY_BREAK					1
#define	ENTRY_REPLACE				1
#define	ENTRY_CONTINUE				0
#define	ENTRY_OK					0

#define ENTRY_MAX					64
#define ENTRY_NONE					((FXint64)0)
#define ENTRY(e)					(((FXint64)1)<<(e))
#define ENTRY_ALL					((FXint64)(0xffffffffffffffff))
//`````````````````````````````````````````````````````````````````````````````



//Access Levels----------------------------------------------------------------
#define ACCESS_WORLD_OBJECT			0x00000004 //receive WO_* world view object messages
#define ACCESS_PROCESS				0x0000000a //receive P_* process messages
#define ACCESS_OBJECT				0x0000000b //receive O_* object messages
#define ACCESS_EFFECT				0x0000000c //receive E_* effect messages
#define ACCESS_WEIGHT				0x00000017 //receive WT_* weight tool messages
#define ACCESS_ARMATURE				0x00000019 //receive ARM_* armature messages
#define ACCESS_ACTION				0x0000001a //receive ACT_* action messages
#define ACCESS_INTERFACE			0x0000001d //receive IN_* interface messages
#define ACCESS_SYSTEM				0x0000001e //receive S_* system messages
#define ACCESS_GENERAL				0x0000001f //receive GEN_* general messages

#define ACCESS_MAX					64 //maximum number of access levels
//`````````````````````````````````````````````````````````````````````````````


//Process Entries--------------------------------------------------------------
#define P_PRE_MOTION				ENTRY(2)  //before motion is calculated
#define P_PRE_IK					ENTRY(4)  //before IK is calculated
#define P_PRE_POINT_DISPLACE		ENTRY(6)  //before point displacement

#define P_POST_MOTION				ENTRY(8)  //after motion is calculated
#define P_POST_IK					ENTRY(10) //after IK is calculated
#define P_POST_POINT_DISPLACE		ENTRY(12) //after point displacement

//Note: The above messages should only be used for effects that need to process at an
//explicity stage.  If you use these messages, be sure to set the FX_MODEFFECT_OVERRIDEPROC
//flag when you register your module.

#define P_DONE						ENTRY(21) //all processing is complete

#define P_ALL						ENTRY_ALL
//`````````````````````````````````````````````````````````````````````````````


//World View Object Specific---------------------------------------------------
#define WO_MOUSEOVER_IN				ENTRY(26) //In Immediate Mode, the mouse has entered the object
#define WO_MOUSEOVER_OUT			ENTRY(27) //In Immediate Mode, the mouse has exited the object

//Note: Although the messages are currently only sent in Immediate, it's possible
//that they may be sent generally in the future.  So, if you use these messages
//don't count on them only being called while in Immediate Mode.

#define WO_ALL						(WO_MOUSEOVER_IN|WO_MOUSEOVER_OUT)
//`````````````````````````````````````````````````````````````````````````````


//Object Entries---------------------------------------------------------------
#define O_CREATE					ENTRY(0) //Object is being created.  Create data for the object, if needed
#define O_INIT						ENTRY(1) //Object is being reinitialized (this can happen as needed within a session)
#define O_DESTROY					ENTRY(2) //Object is being destroyed.  Free the data you created for the object
#define O_COPY						ENTRY(3) //Copy data from one object to another.  Objects are always of the same type
#define O_RENAME					ENTRY(4) //Object has been renamed.

#define O_DESTROY_PRE				ENTRY(5) //Object is about to be destroyed
#define O_COPY_PRE					ENTRY(6) //Object is about to be copied
#define O_RENAME_PRE				ENTRY(7) //Object is about to be renamed

#define O_DESTROY_POST				ENTRY(8) //Object has been destroyed. - YOU WON'T RECEIVE THIS MESSAGE FOR OBJECT BASED MODULES
#define O_COPY_POST					ENTRY(9) //Object has been copied
#define O_RENAME_POST				ENTRY(10) //Object has been renamed

#define O_LOAD						ENTRY(11) //Load data that you saved for the object
#define O_SAVE						ENTRY(12) //Save data for the object

#define O_REPLACE					ENTRY(16) //Object has been replaced - ID will stay the same

#define O_SELECT_PRE				ENTRY(19) //Object is about to be selected
#define O_SELECT_POST				ENTRY(20) //Object has been selected

#define O_DESELECT_PRE				ENTRY(21) //Object is about to be deselected
#define O_DESELECT_POST				ENTRY(22) //Object has been deselected

#define O_SELECT_CHANNEL			ENTRY(23) //Active channel has been changed for the object

#define O_ENTER_SETUP				ENTRY(27) //Setup has been entered for the object
#define O_EXIT_SETUP				ENTRY(28) //Setup has been exited for the object

#define O_MODIFY_REST_PRE			ENTRY(29) //Rest values are about to be modified
#define O_MODIFY_REST_POST			ENTRY(30) //Rest values have been modified

#define O_REPLACE_PRE				ENTRY(31) //Object is about to be replaced 

#define O_PARAM_SET					ENTRY(32) //Set object params - see messiah_interface.h for further info
#define O_PARAM_GET					ENTRY(33) //Get object params - see messiah_interface.h for further info

#define O_ALL						ENTRY_ALL
//`````````````````````````````````````````````````````````````````````````````


//Effect Entries---------------------------------------------------------------
#define E_XFORM_TOOL				ENTRY(7) //Your effect's tool has been moved
#define E_MODIFY_TOOL				ENTRY(8) //Your effect's tool has been modified in some way
#define E_CHANGE_TOOL				ENTRY(9) //Your effect's tool has been changed

#define E_ADD_TARGET				ENTRY(13) //A target has been added to your effect
#define E_DELETE_TARGET				ENTRY(14) //A target has been deleted from your effect
#define E_XFORM_TARGET				ENTRY(15) //Your effect's target has been moved
#define E_MODIFY_TARGET				ENTRY(16) //Your effect's target has been modified in some way
#define E_CHANGE_TARGET				ENTRY(17) //Your effect's target has been changed

#define E_PROCESS					ENTRY(28) //Generalized effect processing (user is in control of process time - preIK, postIK, etc.)

//Note: Unless your effect modifies motion AND displacement or uses a combination of
//ACCESS_PROCESS entries (other than P_DONE), it is highly recommended that you use
//the above entry to process your effect.  This allows the user to have much more
//control over when your effect is processed.

#define E_ALL						ENTRY_ALL
//`````````````````````````````````````````````````````````````````````````````


//Weight Tool Entries----------------------------------------------------------
#define WT_CREATE					ENTRY(0) //Your weight tool is called to create weight data for a target

#define WT_COMPUTE_SPOT				ENTRY(5) //Your weight tool is called to compute weight for a spot

#define WT_ALL						(WT_CREATE|WT_COMPUTE_SPOT)
//`````````````````````````````````````````````````````````````````````````````


//Armature Entries---------------------------------------------------------------
#define ARM_CREATE					ENTRY(0) //Armature has been created
#define ARM_DESTROY					ENTRY(1) //Armature is about to be destroyed
#define ARM_COPY					ENTRY(2) //Armature is being copied

#define ARM_RENAME					ENTRY(3) //Armature has been renamed
#define ARM_SELECT					ENTRY(4) //Armature has been selected

#define ARM_HANDLE_CREATE			ENTRY(5) //Handle has been created
#define ARM_HANDLE_DESTROY			ENTRY(6) //Handle is about to be destroyed
#define ARM_HANDLE_COPY				ENTRY(7) //Handle is being copied

#define ARM_HANDLE_RENAME			ENTRY(8) //Handle has been renamed
#define ARM_HANDLE_SELECT			ENTRY(9) //Handle has been selected
#define ARM_HANDLE_DRAG				ENTRY(10) //Handle is being dragged
#define ARM_HANDLE_DOUBLECLICK		ENTRY(11) //Handle has been double-clicked

#define ARM_ALL						ENTRY_ALL
//`````````````````````````````````````````````````````````````````````````````


//Action Entries---------------------------------------------------------------
//The entries are not currently broadcast to the system, they are sent only to
//the ActionModule that created the Action.
#define ACT_CREATE					ENTRY(0) //Action has been created
#define ACT_DESTROY					ENTRY(1) //Action has been destroyd
#define ACT_COPY					ENTRY(2) //Action has been copied

#define ACT_RENAME					ENTRY(3) //Action has been renamed
#define ACT_SELECT					ENTRY(4) //Action has been selected

#define ACT_SAVE					ENTRY(5) //Save Action data
#define ACT_LOAD					ENTRY(6) //Load Action data

#define ACT_PROCESS					ENTRY(7) //Process the Action

//The messages can only be sent to action modules that register with the FX_MODACTION_MOUSEPROC option.
//Even with that registration flag, you must still request these entries at registration of the module.
#define ACT_MOUSEDRAG				ENTRY(8)
#define ACT_MOUSEDOWN				ENTRY(9)
#define ACT_MOUSEUP					ENTRY(10)
#define ACT_MOUSEDOUBLECLICK		ENTRY(11)
#define ACT_MOUSEIN					ENTRY(12)
#define ACT_MOUSEOUT				ENTRY(13)
#define ACT_LOSEFOCUS				ENTRY(14)
#define ACT_UPDATE					ENTRY(15)


#define ACT_ALL						ENTRY_ALL
//`````````````````````````````````````````````````````````````````````````````



//Interface Entries---------------------------------------------------------------
#define IN_CREATE					ENTRY(0) //Create the interface for your module
#define IN_DESTROY					ENTRY(1) //NOT CURRENTLY SUPPORTED
#define IN_CONFIG					ENTRY(2) //Configure our module's interface

#define IN_ACTIVATE					ENTRY(3) //Your module's interface is about to be activated
#define IN_DEACTIVATE				ENTRY(4) //Your module's interface is about to be deactivated

#define IN_ACTIVATE_POST			ENTRY(5) //Your module's interface has been activated
#define IN_DEACTIVATE_POST			ENTRY(6) //Your module's interface has been deactivated

#define IN_ALL						ENTRY_ALL
//`````````````````````````````````````````````````````````````````````````````


//System Entries---------------------------------------------------------------
#define S_MODULE_INIT				ENTRY(8) //Initialize your module data
#define S_MODULE_UNLOAD				ENTRY(9) //Your module is about to be unloaded


#define S_ALL						(S_MODULE_INIT|S_MODULE_UNLOAD)
//`````````````````````````````````````````````````````````````````````````````


//General Entries--------------------------------------------------------------
#define GEN_GET_GLOBALS				ENTRY(1) //Get the relevant external globals & initialize any data as needed
#define GEN_GET_INTERFACE_GLOBALS	ENTRY(2) //Get the external GUI globals and do any needed interface-specific intialization

#define GEN_CLEAR_SCENE_PRE			ENTRY(3) //Scene is about to be cleared
#define GEN_CLEAR_SCENE_POST		ENTRY(4) //Scene has been cleared
#define GEN_LOAD_SCENE_PRE			ENTRY(5) //Scene is about to be loaded
#define GEN_LOAD_SCENE_POST			ENTRY(6) //Scene has been loaded
#define GEN_APPEND_SCENE_PRE		ENTRY(7) //Scene is about to be appended
#define GEN_APPEND_SCENE_POST		ENTRY(8) //Scene has been appended

#define GEN_LOAD_SCENE				ENTRY(9) //Save scene data
#define GEN_SAVE_SCENE				ENTRY(10) //Load scene data
 
#define GEN_LOAD_CONFIG				ENTRY(11) //Load config file data
#define GEN_SAVE_CONFIG				ENTRY(12) //Save config file data

#define GEN_LOAD_CONFIG_POST		ENTRY(13) //Config file has been loaded
#define GEN_SAVE_CONFIG_POST		ENTRY(14) //Config file has been saved



#define GEN_ALL						ENTRY_ALL
//`````````````````````````````````````````````````````````````````````````````




#endif //_MESSIAH_ACCESS

