/*=============================================================================
	Copyright (c) 1999-2003 pmG Worldwide, LLC.

	File:	messiah_data.h
	Author:	Lyle Milton
	Date:	01/01/03

		last modified: 2/04/03 - LM

	------------------------------------------------------------

	The data management functions allow you to store data with the
	supported types: 
			FXobject	//Universal Object.  Generally used to represent geometric objects or any other "Object based" ID
			FXtool		//Tool.  Object based
			FXeffect	//Effect. Object based
			FXshader	//Shader. Object based
			FXcontrol	//Control.  GUI control
			FXimage		//Image

	The advantages of using the data management system is that
	your data will be "attached" to the entity with which you
	created the data.  Also, the data is automatically freed
	when the entity is destroyed (provided you used fxDataCreate,
	and not fxDataSet).

	The data system uses a key & keyID to uniquely identify your
	stored data.  The key & keyID are completely arbitrary are
	define by you.  For example, if you need to store data with a
	target object for effect calculation, you could do this:


		FXdouble	*data;
		-
		-
		-
		data = fxDataCreate(target, &effect_module_ID, 0, sizeof(FXdouble));

	In this example, the effect module's ID is used as the key, and 0 is used
	as the keyID.  And you would retrieve that data like so:

		data = fxDataGet(target, &effect_module_ID, 0);

	If you needed to store multiple pieces of data with the target(s)
	using that same key, you could do so by using different keyIDs with that
	same key:

		#define MYVALUE		0
		#define MYSTRUCT	1
		-
		-
		-
		mval = fxDataCreate(target, &effect_module_ID, MYVALUE, sizeof(FXdouble));
		data = fxDataCreate(target, &effect_module_ID, MYSTRUCT, sizeof(MyStruct));

	This is fine if you're storing data based on the effect *module* but
	if you need to store data based on the effect *object* (instance),
	it may not be practical to use the module's ID, you need to use a key
	that would be unique to that effect object.  There are 2 easy ways to
	do this.

	The first method is to use the "typedata" from the effect object as
	the key:

		tdat = fxObjectGetTypeData(effect, 0);
		data = fxDataCreate(target, tdat, 0, sizeof(FXdouble));

	That method is safe, but if your effect doesn't need any typedata,
	you'll need to use the second method.

	The second method is to use the effect ID as the keyID.  This means
	that you'll need to use some other data as the key.  The key can be
	anything; it could even be the address of the effect module's ID:

		data = fxDataCreate(target, &effect_module_ID, effect, sizeof(FXdouble));

	The only problem with this method is that you can't store multiple
	pieces of data as previously described.  If this is needed, you can do
	something like this:

		char	*my_value = "MyValue"; //must be global
		char	*my_struct = "MyStruct"; //must be global
		-
		-
		-
		mval = fxDataCreate(target, my_value, effect, sizeof(FXdouble));
		data = fxDataCreate(target, my_struct, effect, sizeof(MyStruct));

	In that example, the address of the strings are used as keys.  The
	actual contents of the stings are irrelevant.  However, if you need
	to use "named" data, you can use a special keyID:

		mval = fxDataCreate(target, my_value, FX_DATAKEY_NAME, sizeof(FXdouble));
		data = fxDataCreate(target, my_struct, FX_DATAKEY_NAME, sizeof(MyStruct));

	Using FX_DATAKEY_NAME tells the data manager to create/retrieve
	data based on the text in the string key.	!!!CAUTION!!!! only
	pass NULL terminated strings as keys when you use FX_DATAKEY_NAME.


=============================================================================*/

#ifdef _MESSIAH_API_DATA

#ifndef __MESSIAH_DATA
#define __MESSIAH_DATA

#ifndef __MESSIAH_HIDE_TYPES

	#define FX_DATAKEY_NAME		-1

#endif //__MESSIAH_HIDE_TYPES


FX_API(FXvoid*, fxDataCreate)(FXentity, FXvoid *key, FXint keyID, FXulong size);

FX_API(FXvoid*, fxDataSet)(FXentity, FXvoid *key, FXint keyID, FXvoid *data);

FX_API(FXvoid*, fxDataGet)(FXentity, FXvoid *key, FXint keyID);

FX_API(FXint, fxDataFree)(FXentity, FXvoid *key, FXint keyID);

FX_API(FXint, fxDataFreeKey)(FXentity, FXvoid *key);

#define _MESSIAH_PTR_DATA	messiah_apicomponent = FX_APIFUNCS_DATA;\
							FX_PTR(fxDataCreate);\
							FX_PTR(fxDataSet);\
							FX_PTR(fxDataGet);\
							FX_PTR(fxDataFree);\
							FX_PTR(fxDataFreeKey)

#endif //__MESSIAH_DATA

#endif //_MESSIAH_API_DATA
