/*=============================================================================
	Copyright (c) 1999-2004 pmG Worldwide, LLC.

	File:	messiah_utility.h
	Author:	Lyle Milton
	Date:	06/10/04

		last modified: 06/10/04 - LM

	-----------------------------------------------------------------
	ABORTING PROCESSES
	-----------------------------------------------------------------
	It's typical for some modules to perform very time consuming
	operations.  In these situations, the user may not always be have
	the luxury of waiting for a module to finish, and will need to 
	cancel the operation.  For example, your Edit module may perform
	some compute intensive motion editing operation on multiple items:
	
	EX1: Processing Loop
	
		for (i=0;i<num_objects;i++)
		{
			//perform motion processing.
		}

	In that loop, you could be performing any number of operations on
	an large number of objects.  Since this loop is local to your
	module but resides in the main program thread, messiah will appear
	to be locked until your process ends.  Even if you use
	fxMessageProgress the user still may need to cancel the operation.
	Fortunately the API has a very simple way to accomplish this.

	EX2: Checking for User Abort
	
		for (i=0;i<num_objects;i++)
		{
			if (fxProcessAbort("Motion Process", NULL, FX_NULLID, FX_NULLID, NULL, NULL, 0)) break;

			//perform motion processing.
		}

	By inserting a fxProcessAbort call in your loop, you're giving the
	user the chance to use the Esc key to abort.  If the user chooses
	to abort the fxProcessAbort function will return FX_ABORT_TRUE
	Nothing to it.
	
	When the user hits the Esc key, a standard message will be posted
	asking for abort confirmation.  You can customize the message to
	something more meaningful by using the *message param.


	EX3: Checking for User Abort
	
		for (i=0;i<num_objects;i++)
		{
			if (fxProcessAbort("Motion Process",
					"Aborting this operation will leave some items unedited.\nAbort the operation anyway?",
					FX_NULLID, FX_NULLID, NULL, NULL, 0)) break;

			//perform motion processing.
		}

	-----------------------------------------------------------------

	There may be times when you may need to perform other checks or
	post other info before the user decides to abort.  In this case you
	can completely override the message box to create your own and/or
	perform some other useful operations for the user.  First you'd need
	to define an AbortFunc.

	EX4: Defining An Abort Callback

		FXintf AbortFunc(FXmodule moduleID, FXcontrol conID, My_AbortData *data, FXint flags)

		-OR-

		FX_ABORTFUNC(AbortFunc, My_AbortData, data)

		{
			FXint		abort = 0;
			//todo: perform some useful op and/or give the user a chance to abort
			-
			-
			-
			if (abort)
				return FX_ABORT_TRUE;
			else
				return FX_ABORT_FALSE;
		}

	With your Abort callback  defined, you're now ready to perform abort
	testing.


	EX5: Checking for User Abort
	
		for (i=0;i<num_objects;i++)
		{
			if (fxProcessAbort(NULL, NULL, FX_NULLID, FX_NULLID, data, &AbortFunc, 0)) break;

			//perform motion processing.
		}

	With that configuration, when the user hits the Esc key, no message
	will be automatically posted, your Abort callback will be called
	instead. You are in control of aborting by returning FX_ABORT_TRUE or
	FX_ABORT_FALSE based on user response to info you post or other data.

	If you're performing muliple time-consuming operations in that loop, it's
	perfectly fine to call fxProcessAbort more than once.


	EX6: Checking for User Abort
	
		for (i=0;i<num_objects;i++)
		{
			if (fxProcessAbort(NULL, NULL, FX_NULLID, FX_NULLID, data, &AbortFunc, 0)) break;

			//perform motion processing.

			if (fxProcessAbort(NULL, NULL, FX_NULLID, FX_NULLID, data, &AbortFunc, 0)) break;

			//perform some other time consuming stuff.

			if (fxProcessAbort(NULL, NULL, FX_NULLID, FX_NULLID, data, &AbortFunc, 0)) break;

			//blah, blah, blah
		}

	-----------------------------------------------------------------
	NOTE:  If you abort a process, you are in charge of cleaning up
	your own data and restoring the original state of your data, if possible.
	

=============================================================================*/

#ifdef _MESSIAH_API_UTILITY

#ifndef __MESSIAH_UTILITY
#define __MESSIAH_UTILITY


#ifndef __MESSIAH_HIDE_TYPES

	//return values from fxProcessAbort,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
	#define		FX_ABORT_TRUE		FX_TRUE
	#define		FX_ABORT_FALSE		FX_FALSE
	//`````````````````````````````````````````````````````````````````````````

	#define FX_ABORTFUNC(f, dt, ed)	FXintf f(FXmodule moduleID, FXcontrol conID, dt *ed, FXint flags)


	#define FX_PATH_ABSOLUTE			(1<<0)

#endif //__MESSIAH_HIDE_TYPES


//ID's for use with fxPluginPath,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
#define FX_PATHID_PLUGINS			1 //Plugins directory
#define FX_PATHID_SCENEFILE			2 //Scene file name
#define FX_PATHID_CONFIG			3 //messiah temp (or config) directory
#define FX_PATHID_CONTENT			4 //content directory
#define FX_PATHID_SCENECONTENT		5 //content directory as loaded from scene
#define FX_PATHID_PROGRAM			6 //the messiah program directory
#define FX_PATHID_SYSTEMP			7 //your system's temp directory
#define FX_PATHID_MODULES			8 //the messiah modules directory
//`````````````````````````````````````````````````````````````````````````````


//for use with UNDO SYSETM.
//your undo function will be called with these notifications
#define FX_UOP_RECORD_UNDO		0
#define FX_UOP_RECORD_REDO		1
#define FX_UOP_UNDO				2
#define FX_UOP_REDO				3
#define FX_UOP_DELETE_UNDO		4
#define FX_UOP_DELETE_REDO		5

#define FX_UNDO_RESTART			(1<<0)
#define FX_UNDO_RECORDED		(1<<1)
#define FX_UNDO_RENAME			(1<<2)
#define FX_UNDO_FORCERECORD		(1<<3)
//```````````````````````````````````````````````````````




FX_API(FXuint, fxDongleID)(FXvoid);

FX_API(FXint, fxPathGet)(FXint pathID, FXchar *buffer, FXint buffer_length, FXint flags);

FX_API(FXint, fxPluginPath)(FXchar *buffer, FXint buffer_length, FXint flags);

FX_API(FXint, fxProcessAbort)
(
	FXchar		*title,
	FXchar		*message,
	FXmodule	moduleID,
	FXcontrol	conID,
	FXvoid		*data,
	FXint		(*AbortFunc)(FXmodule, FXcontrol, FXvoid *data, FXint flags),
	FXint		flags
);


//UNDO SYSTEM////////////////////////////
//fxUndoTypeRegister("myundofuncname", &MyCallBackFunction, sizeof(MyCustomData_if_needed), 0);
FX_API(FXint, fxUndoTypeRegister)
(
	FXchar *undo_name, 
	FXint (*callback_func)(void *system_handle, FXint holder_id, void *your_custom_data, void *item_handle, int undo_opearation), 
	FXuint data_size, //optional if you want custom data that you can access later when called
	FXint flags
);
FX_API(FXvoid, fxUndoBegin)(FXchar *label, FXint flags);
//holder_id is what thing will hold onto this undo. It's like an association with an object or effect
FX_API(FXint, fxUndoRecord)(FXint holder_id, FXchar *undo_name);
FX_API(FXvoid, fxUndoEnd)(FXvoid);

//USEAGE
//make new undo entry
/*
fxUndoBegin("mylabel_for_this_undo",FX_UNDO_RENAME); 
fxUndoRecord(objectid, "myundofuncname"); //tells your function when it's time to save the current state

	//do whatever actions and change whatever data you want here

//tell the system you're done with this state
fxUndoEnd();
*/
///////////////////////////////////////////


#define _MESSIAH_PTR_UTILITY	messiah_apicomponent = FX_APIFUNCS_UTILITY;\
								FX_PTR(fxDongleID);\
								FX_PTR(fxPathGet);\
								FX_PTR(fxPluginPath);\
								FX_PTR(fxProcessAbort);\
								FX_PTR(fxUndoBegin);\
								FX_PTR(fxUndoEnd);\
								FX_PTR(fxUndoTypeRegister);\
								FX_PTR(fxUndoRecord)


#endif //__MESSIAH_UTILITY

#endif //_MESSIAH_API_UTILITY
