/*=============================================================================
	Copyright (c) 1999-2004 pmG Worldwide, LLC.

	File:	messiah_effect.h
	Author:	Lyle Milton
	Date:	04/25/04

		last modified: 04/25/04 - LM


	-----------------------------------------------------------------
	CREATING WEIGHT TOOLS
	-----------------------------------------------------------------
	Much like Procedural or MetaEffector, you can create your own
	weight tools for general use throught messiah.  The major difference
	is that your tools can not be as complex as a tool like MetaEffector
	at this time.  However, it is possible to create some very powerful
	weight tools. For example, you can create a tool that, say,
	generates a noise feild or imports weight maps from other sources.

	Setting up a weight tool really isn't any different from creating
	any other object type like effects or shaders.  You just need to
	set the appropriate data when defining the module.

	EX1: Defining a Weight Tool

		_pluginEntry
		{
			-
			-
			-
			fxModuleRegister(FX_MODTYPE_TOOL, "MyWeightModule", &WeightAccess, FX_MODTOOL_WEIGHT);
			-
			-
			-

	So, to define a weight tool module, just set the module type to
	FX_MODTYPE_TOOL and set the flags to FX_MODTOOL_WEIGHT.  Your tool
	will not only be available in the Tools list, it will also be
	available in the Weight list.


	-----------------------------------------------------------------
	PROCESSING TARGET WEIGHTS
	-----------------------------------------------------------------
	There are essentially only two instances when your weight tool module
	may be called upon to generate weights.  Your module will be called
	to generate weights for an entire target, or for an arbitrary 
	position in 3D space.  So, you must be prepared to compute weight
	data for both of these instances as they apply to your weight tool
	module.

	However, before you can be called to process weight data, you'll need
	to set the appropriate access for your module.

	EX2: Setting Access for a Weight Tool

		_pluginEntry
		{
			-
			-
			-
			fxModuleAccess(ACCESS_WEIGHT, WT_CREATE|WT_COMPUTE_SPOT, 0);
			fxModuleRegister(FX_MODTYPE_TOOL, "MyWeightModule", &WeightAccess, FX_MODTOOL_WEIGHT);
			-
			-
			-

	In the above example, the WT_CREATE message corresponds to target
	weighting and the WT_COMPUTE_SPOT message corresponds to spot weighting.
	Once that's all set up, you're ready to define your processing code.

	One way to perform target processing is to scan through the target's
	points, and provide a weight value for each point.

	EX3: Target Weight Processing

		case WT_CREATE:
			-
			-
			-
			fxWeightToolData(&target, &weight_type, 0);

			if (weight_type==FX_WEIGHTTYPE_ANIMATE) point_type = FX_MESHDATA_WORLD;
			else point_type = FX_MESHDATA_OBJ;

			fxMeshNumPoints(target, &num_points, 0);

			for (i=0;i<num_points;i++)
			{
				fxMeshPoint(target, i, pos, point_type);

				//todo: compute some weight value
				-
				-
				-
				fxWeightToolValueSet(i, computed_weight_value, 0);
			}

	From the above code, the first step is to get the target and weight
	type via the fxWeightToolData funtion.  The weight_type corresponds
	to the user's chosen weight type on the effect's interface or
	weightdraw list.  You'd use this value to determine which target
	points should be used to compute the weight data.  Since SETUP and
	COMBO weight types both use the original "as loaded" points for the
	target, it's only necessary to check for ANIMATE as demonstrated
	above.

	The rest of the code just does simple mesh scanning for the target
	to get the point position to calculate weights.  Once the 
	weight is calculated for a point, all that is necessary is a call to
	fxWeightToolValueSet, which does just what it's name suggests.

	Scanning the target yourself is useful when you need direct control
	over which points are processed and when.  However, if you don't
	need such control, the API has a mechanism to perform the scan for
	you.

	You first need to define your scanning func like so...

	EX4: Target Weight Scanning Callback

		MyWTScanFunc(FXtool toolID, FXobject targetID, FXint pointID, FXdouble pos[3], FXfloat *weight, MyWTScanData *data)

		OR

		FX_WEIGHTTOOLSCAN(MyWTScanFunc, MyWTScanData, data)

		{
			//todo: calculate weight data and put it to FXfloat *weight

			return FX_TRUE;
		}

	Then you can perform your scan:

	EX5: Scanning a Target to Compute Weight Data

		case WT_CREATE:

			MyWTScanData  data;

			//initialize your data...
			-
			-
			-
			fxWeightToolScan(toolID, &data, &MyWTScanFunc, 0);

	Note that EX5 requires less code than EX3 because the target & 
	weight type retrieval are not necessary.  Also, the appropriate
	point data is sent to your callback, so it's not necessary for
	you to get this data yourself.


	-----------------------------------------------------------------
	PROCESSING A SPOT WEIGHT
	-----------------------------------------------------------------
	When you receive a WT_COMPUTE_SPOT spot message, you'll must
	process the spot passed to your Access func.  Because there is no
	target associated with this spot, you'll need to retreive the spot
	from AccessInfo.

	EX6: Spot Weight Processing

		case WT_COMPUTE_SPOT:
			-
			-
			-
			fxWeightToolPos(ai, pos, 0);

			//todo: compute some weight value based on that spot
			-
			-
			-
			fxWeightToolSpotSet(ai, computed_weight_value, 0);

	As you can see, there's not much to worry about with spot processing.
	There are only 2 functions involved: fxWeightToolPos to get the
	position to be processed, and fxWeightToolSpotSet to set the weight
	for that spot.


	-----------------------------------------------------------------

	IMPORTANT NOTE: Any attempt to process weight data outside of the
	context of the weight messages will fail.  For example, you shouldn't
	attempt to modify weight data of *any* targets in response to a 
	P_PRE_POINT_DISPLACE message.  The user is always in control of the
	application of weights via the chosen weight type on the relevant
	interfaces.  If the user wants animated weights, she will set the
	weight type to ANIMATE/Dynamic, and you'll be notified to process
	weights as needed during an animation.


	-----------------------------------------------------------------
	WEIGHTDRAW BLOCK
	-----------------------------------------------------------------
	When using tools such as Curve, Procedural, etc, you'll notice that
	they have a block called "Weight List".  This block is designed to
	allow the user to determine which targets/tools will be used for
	weight drawing in the world views.  If you're creating a weight
	tool, you must provide this functionality, as well.  Forturnately,
	the API takes care of this for you via the WeightDraw block.

	EX7: Accessing the WeightDraw Block

		case IN_CREATE:
			-
			-
			-
			fxInterfaceBlockSet(FX_MODE_ANIMATE, FX_MODE_ANIMATE, FX_MBLOCK_WEIGHTDRAW, FX_IDNULL, 0);
			-
			-
			-
			fxInterfaceBlockSet(FX_MODE_SETUP, FX_MODE_SETUP, FX_MBLOCK_WEIGHTDRAW, FX_IDNULL, 0);


	As demonstrated in EX7, to allow your weight tool module to use the
	WeightDraw block, you only need to call fxInterfaceBlockSet for your
	Animate and Setup interfaces when you receive your IN_CREATE message.
	Once the user accesses your interface he will be presented with the
	"Weight List" block and all functionality is handled automatically.  
	See messiah_interface.h for more details on configuring your interfaces 
	with fxInterfaceBlockSet;

	

=============================================================================*/

#ifdef _MESSIAH_API_WEIGHTTOOL

#ifndef __MESSIAH_WEIGHTTOOL
#define __MESSIAH_WEIGHTTOOL

#ifndef __MESSIAH_HIDE_TYPES

	#define FX_WEIGHTTYPE_SETUP		(1) //Compute map from setup (will only be updated while in Setup)
	#define FX_WEIGHTTYPE_ANIMATE	(2) //Compute map from animate (dynamic map) - will be updated as needed in any mode.
	#define FX_WEIGHTTYPE_COMBO		(4) //Compute map based on original (as loaded) target points using current weight motion regardless of mode

	#define FX_WEIGHTTOOLSCAN(f, dt, ed)	FXintf f(FXtool toolID, FXobject targetID, FXint pointID, FXdouble pos[3], FXfloat *weight, dt *ed)

#endif //__MESSIAH_HIDE_TYPES


FX_API(FXint, fxWeightToolData)(FXobject *targetID, FXint *weight_type, FXint flags);

FX_API(FXint, fxWeightToolValue)(FXint pointID, FXfloat *weight, FXint flags);

FX_API(FXint, fxWeightToolValueSet)(FXint pointID, FXfloat weight, FXint flags);

FX_API(FXint, fxWeightToolScan)
(
	FXint	toolID,
	FXvoid	*data,
	FXint	(*WTScan)(FXtool toolID, FXobject targetID, FXint pointID, FXdouble pos[3], FXfloat *weight, FXvoid *data),
	FXint	flags
);

FX_API(FXint, fxWeightToolPos)(FX_AccessInfo *ai, FXdouble pos[3], FXint flags);

FX_API(FXint, fxWeightToolSpotSet)(FX_AccessInfo *ai, FXfloat weight, FXint flags);

FX_API(FXint, fxWeightTargetNext)(FXint toolID, FXobject weight_target_curr);


#define _MESSIAH_PTR_WEIGHTTOOL	messiah_apicomponent = FX_APIFUNCS_TOOLTYPE;\
							FX_PTR(fxWeightToolData);\
							FX_PTR(fxWeightToolValue);\
							FX_PTR(fxWeightToolValueSet);\
							FX_PTR(fxWeightToolScan);\
							FX_PTR(fxWeightToolPos);\
							FX_PTR(fxWeightToolSpotSet);\
							FX_PTR(fxWeightTargetNext)

#endif //__MESSIAH_WEIGHTTOOL

#endif //_MESSIAH_API_WEIGHTTOOL
